<?php
/*
Plugin Name: Cleanup Category & Tags (EasyToolsWP)
Description: Manually clean up empty categories and tags for posts and products via WordPress settings, set auto-delete options, and redirect directly to the product or post if there's only one in a category/tag.
Version: 1.7
Author: EasyToolsWP.com
*/

// Prevent direct access
if (!defined('ABSPATH')) {
    exit;
}

// 1. Create a settings page with manual cleanup buttons, auto-delete settings, and redirection settings
add_action('admin_menu', 'cleanup_settings_menu');

function cleanup_settings_menu() {
    add_options_page(
        'Cleanup Settings',           // Page title
        'Cleanup Categories & Tags',  // Menu title
        'manage_options',             // Permission
        'cleanup-settings',           // Slug for the page
        'cleanup_settings_page'       // Callback to display the page
    );
}

function cleanup_settings_page() {
    ?>
    <div class="wrap">
        <h1>Cleanup Categories and Tags Settings</h1>
        <form method="post" action="">
            <?php wp_nonce_field('cleanup_nonce_action', 'cleanup_nonce_field'); ?>

            <h2>Manual Cleanup</h2>
            <p>Click the button below to delete empty categories or tags (those that have no associated posts or products).</p>

            <?php
            if (isset($_POST['delete_empty_categories'])) {
                if (check_admin_referer('cleanup_nonce_action', 'cleanup_nonce_field')) {
                    $deleted_count = cleanup_empty_terms('category');
                    echo '<div class="notice notice-success"><p>' . $deleted_count . ' empty post categories have been deleted.</p></div>';
                } else {
                    echo '<div class="notice notice-error"><p>Verification failed. Please try again.</p></div>';
                }
            }

            if (isset($_POST['delete_empty_tags'])) {
                if (check_admin_referer('cleanup_nonce_action', 'cleanup_nonce_field')) {
                    $deleted_count = cleanup_empty_terms('post_tag');
                    echo '<div class="notice notice-success"><p>' . $deleted_count . ' empty post tags have been deleted.</p></div>';
                } else {
                    echo '<div class="notice notice-error"><p>Verification failed. Please try again.</p></div>';
                }
            }

            if (isset($_POST['delete_empty_product_categories']) && class_exists('WooCommerce')) {
                if (check_admin_referer('cleanup_nonce_action', 'cleanup_nonce_field')) {
                    $deleted_count = cleanup_empty_terms('product_cat');
                    echo '<div class="notice notice-success"><p>' . $deleted_count . ' empty product categories have been deleted.</p></div>';
                } else {
                    echo '<div class="notice notice-error"><p>Verification failed. Please try again.</p></div>';
                }
            }

            if (isset($_POST['delete_empty_product_tags']) && class_exists('WooCommerce')) {
                if (check_admin_referer('cleanup_nonce_action', 'cleanup_nonce_field')) {
                    $deleted_count = cleanup_empty_terms('product_tag');
                    echo '<div class="notice notice-success"><p>' . $deleted_count . ' empty product tags have been deleted.</p></div>';
                } else {
                    echo '<div class="notice notice-error"><p>Verification failed. Please try again.</p></div>';
                }
            }
            ?>

            <p>
                <input type="submit" name="delete_empty_categories" class="button button-primary" value="Delete Empty Post Categories">
                <input type="submit" name="delete_empty_tags" class="button button-primary" value="Delete Empty Post Tags">
                <?php if (class_exists('WooCommerce')): ?>
                    <input type="submit" name="delete_empty_product_categories" class="button button-primary" value="Delete Empty Product Categories">
                    <input type="submit" name="delete_empty_product_tags" class="button button-primary" value="Delete Empty Product Tags">
                <?php endif; ?>
            </p>

            <h2>Auto-Delete Settings</h2>
            <p>Set the number of days for automatic deletion of empty categories and tags.</p>

            <?php
            if (isset($_POST['save_auto_delete_settings'])) {
                if (check_admin_referer('cleanup_nonce_action', 'cleanup_nonce_field')) {
                    update_option('cleanup_interval_days', intval($_POST['cleanup_interval_days']));
                    update_option('cleanup_auto_delete_categories', isset($_POST['cleanup_auto_delete_categories']));
                    update_option('cleanup_auto_delete_tags', isset($_POST['cleanup_auto_delete_tags']));
                    update_option('cleanup_auto_delete_product_categories', isset($_POST['cleanup_auto_delete_product_categories']));
                    update_option('cleanup_auto_delete_product_tags', isset($_POST['cleanup_auto_delete_product_tags']));
                    update_option('redirect_if_one_post_or_product', isset($_POST['redirect_if_one_post_or_product']));
                    cleanup_schedule_event();
                    echo '<div class="notice notice-success"><p>Auto-delete settings and redirect settings have been saved.</p></div>';
                }
            }

            // Load current settings
            $interval_days = get_option('cleanup_interval_days', 14);
            $auto_delete_categories = get_option('cleanup_auto_delete_categories', false);
            $auto_delete_tags = get_option('cleanup_auto_delete_tags', false);
            $auto_delete_product_categories = get_option('cleanup_auto_delete_product_categories', false);
            $auto_delete_product_tags = get_option('cleanup_auto_delete_product_tags', false);
            $redirect_if_one_post_or_product = get_option('redirect_if_one_post_or_product', false);
            ?>

            <p>
                <label for="cleanup_interval_days">Days to Auto-Delete:</label>
                <input type="number" id="cleanup_interval_days" name="cleanup_interval_days" value="<?php echo esc_attr($interval_days); ?>" min="1">
            </p>

            <p>
                <input type="checkbox" id="cleanup_auto_delete_categories" name="cleanup_auto_delete_categories" <?php checked($auto_delete_categories); ?>>
                <label for="cleanup_auto_delete_categories">Auto-delete empty post categories</label>
            </p>

            <p>
                <input type="checkbox" id="cleanup_auto_delete_tags" name="cleanup_auto_delete_tags" <?php checked($auto_delete_tags); ?>>
                <label for="cleanup_auto_delete_tags">Auto-delete empty post tags</label>
            </p>

            <?php if (class_exists('WooCommerce')): ?>
                <p>
                    <input type="checkbox" id="cleanup_auto_delete_product_categories" name="cleanup_auto_delete_product_categories" <?php checked($auto_delete_product_categories); ?>>
                    <label for="cleanup_auto_delete_product_categories">Auto-delete empty product categories</label>
                </p>

                <p>
                    <input type="checkbox" id="cleanup_auto_delete_product_tags" name="cleanup_auto_delete_product_tags" <?php checked($auto_delete_product_tags); ?>>
                    <label for="cleanup_auto_delete_product_tags">Auto-delete empty product tags</label>
                </p>
            <?php endif; ?>

            <h2>Redirection Setting</h2>
            <p>Enable redirection if a category/tag contains only one post or product.</p>
            <p>
                <input type="checkbox" id="redirect_if_one_post_or_product" name="redirect_if_one_post_or_product" <?php checked($redirect_if_one_post_or_product); ?>>
                <label for="redirect_if_one_post_or_product">Redirect directly to post/product if there's only one in the category/tag</label>
            </p>

            <p>
                <input type="submit" name="save_auto_delete_settings" class="button button-primary" value="Save Auto-Delete and Redirection Settings">
            </p>
        </form>
    </div>
    <?php
}

// 2. General function to delete empty terms
function cleanup_empty_terms($taxonomy) {
    $terms = get_terms(array(
        'taxonomy' => $taxonomy,
        'hide_empty' => false,
        'fields' => 'ids',
    ));

    $deleted_count = 0;
    if (!empty($terms) && !is_wp_error($terms)) {
        foreach ($terms as $term_id) {
            $term = get_term($term_id, $taxonomy);
            if ($term->count == 0) {
                wp_delete_term($term_id, $taxonomy);
                $deleted_count++;
            }
        }
    }

    return $deleted_count;
}

// 3. Schedule auto-delete event
function cleanup_schedule_event() {
    // Clear existing scheduled events
    wp_clear_scheduled_hook('cleanup_auto_delete_event');

    // Schedule new event based on user setting
    $interval_days = get_option('cleanup_interval_days', 14);
    if ($interval_days > 0) {
        wp_schedule_event(time(), 'daily', 'cleanup_auto_delete_event');
    }
}

// 4. Auto-delete hook
add_action('cleanup_auto_delete_event', 'cleanup_auto_delete_handler');

function cleanup_auto_delete_handler() {
    if (get_option('cleanup_auto_delete_categories')) {
        cleanup_empty_terms('category');
    }
    if (get_option('cleanup_auto_delete_tags')) {
        cleanup_empty_terms('post_tag');
    }
    if (class_exists('WooCommerce')) {
        if (get_option('cleanup_auto_delete_product_categories')) {
            cleanup_empty_terms('product_cat');
        }
        if (get_option('cleanup_auto_delete_product_tags')) {
            cleanup_empty_terms('product_tag');
        }
    }
}

// 5. Redirect to post or product if one exists in category/tag
add_action('template_redirect', 'redirect_to_single_post_or_product');

function redirect_to_single_post_or_product() {
    if (get_option('redirect_if_one_post_or_product')) {
        if (is_category() || is_tag()) {
            $term = get_queried_object();
            if ($term && !is_wp_error($term)) {
                $query_args = array(
                    'post_type' => 'post',
                    'tax_query' => array(
                        array(
                            'taxonomy' => $term->taxonomy,
                            'field'    => 'term_id',
                            'terms'    => $term->term_id,
                        ),
                    ),
                    'posts_per_page' => 1,
                );
                $query = new WP_Query($query_args);
                if ($query->have_posts() && $query->found_posts == 1) {
                    wp_redirect(get_permalink($query->posts[0]->ID));
                    exit;
                }
            }
        }

        if (class_exists('WooCommerce') && (is_product_category() || is_product_tag())) {
            $term = get_queried_object();
            if ($term && !is_wp_error($term)) {
                $query_args = array(
                    'post_type' => 'product',
                    'tax_query' => array(
                        array(
                            'taxonomy' => $term->taxonomy,
                            'field'    => 'term_id',
                            'terms'    => $term->term_id,
                        ),
                    ),
                    'posts_per_page' => 1,
                );
                $query = new WP_Query($query_args);
                if ($query->have_posts() && $query->found_posts == 1) {
                    wp_redirect(get_permalink($query->posts[0]->ID));
                    exit;
                }
            }
        }
    }
}

// 6. Activation and Deactivation Hooks
register_activation_hook(__FILE__, 'cleanup_schedule_event');
register_deactivation_hook(__FILE__, function() {
    wp_clear_scheduled_hook('cleanup_auto_delete_event');
});
